#!/usr/bin/env python3

# Resol-Reader by Oliver Schindler
# reads values from Resol-Solar control and sends it to middleware of Volkszaehler
# https://www.volkszaehler.org/
# makes use of resol-decoder library from Nicola Heile

import time
import sys
import resol
import requests
import serial
import signal

# Load settings
try:
    import config
except:
    sys.exit("config.py not found!")

run_loop = True
ser = None

# some recorded data for testing
# recorded message, with Temp1 = 12.2 , Temp2 = 9.8 , Drehzahl = 0
# recorded message, with Temp1 = 55.1 , Temp2 = 13.2 , Drehzahl = 100

if config.simulation:
    messages_array = [[[170, 16, 0, 123, 66, 16, 0, 1, 9, 24, 122, 0, 98, 0, 0, 35, 98, 0, 56, 34, 4, 63, 0, 127, 0, 0, 2, 126, 0, 127, 0, 0, 2, 126, 11, 1, 127, 127, 12, 105, 0, 0, 16, 3, 4, 104, 0, 0, 0, 0, 0, 127, 0, 0, 0, 0, 0, 127, 104, 0, 0, 0, 0, 23]],
                     [[170, 16, 0, 123, 66, 16, 0, 1, 9, 24, 122, 0, 98, 0, 0, 35, 98, 0, 56, 34, 4, 63, 0, 127, 0, 0, 2, 126, 0, 127, 0, 0, 2, 126, 11, 1, 127, 127, 12, 105, 0, 0, 16, 3, 4, 104, 0, 0, 0, 0, 0, 127, 0, 0, 0, 0, 0, 127, 104, 0, 0, 0, 0, 23]],
                     [[170, 16, 0, 123, 66, 16, 0, 1, 9, 24, 122, 0, 98, 0, 0, 35, 98, 0, 56, 34, 4, 63, 0, 127, 0, 0, 2, 126, 0, 127, 0, 0, 2, 126, 11, 1, 127, 127, 12, 105, 0, 0, 16, 3, 4, 104, 0, 0, 0, 0, 0, 127, 0, 0, 0, 0, 0, 127, 104, 0, 0, 0, 0, 23]],
                     [[170, 16, 0, 123, 66, 16, 0, 1, 9, 24, 39, 2, 4, 0, 4, 78, 2, 0, 56, 34, 5, 30, 100, 127, 5, 0, 2, 21, 0, 127, 0, 0, 2, 126, 11, 1, 127, 127, 12, 105, 0, 0, 3, 3, 4, 117, 0, 0, 0, 0, 0, 127, 0, 0, 0, 0, 0, 127, 104, 0, 0, 0, 0, 23]]]

def signal_handler(sig, frame):
    global run_loop
    print('You pressed Ctrl+C!')
    run_loop = False
    #sys.exit(0)
    
# Logs in onto the DeltaSol BS Plus over LAN. Also starts (and maintains) the
# actual stream of data.
def login(): # From Network Login - For serial only flush input Buffer and test connection
    try:
        dat = recv()

        return True
    except:
        return False

# Receive 1024 bytes from stream
def recv():
    global ser
    global run_loop
    while run_loop:
        dat = ser.read(1)
        if dat:
            break
        if config.debug:
            print ("nothing received")
    ret = int('0x' + dat.hex(),16)
    return ret


# Read Data until minimum 6 message is received
def readstream():
    global run_loop
    data = [recv()]
    while (data.count((0xAA)) < 6) and run_loop:
        data.append(recv())
    return data


def parse_values(msg): # Pick my Values

    act_values = []
    frame = resol.get_payload(msg)
    for channel in  range(len (config.channels)):
        framenumber = config.channels[channel][1]
        offset = config.channels[channel][2]
        length = config.channels[channel][3]
        divisor = config.channels[channel][4]
        if config.debug:
            print ("Reading frame ",framenumber," offset ",offset," length ",length," divisor ",divisor)
        if frame[framenumber] != False:            
            value = resol.gb(frame[framenumber], offset, offset+length) / divisor
            act_values.append(value)
            if config.debug:
                print("readout value [",channel,"] = " + str(value))

    return act_values   

def load_data():

    global message_count
    
    if not config.simulation:
        buf = readstream()
        messages = resol.splitmsg(buf)
        messages = resol.selectDestination(messages, '0x0010') # 0x0010 0xFFFF DFA adressed
    else:
        messages = messages_array[message_count]
            

    if config.debug:
        print("messages = " + str(messages))
    
    for msg in messages:
        if "PV1" == resol.get_protocolversion(msg):
            if config.debug & False:
                print(resol.format_message_pv1(msg))
            act_values = parse_values(msg)
        elif "PV2" == resol.get_protocolversion(msg):
            if config.debug:
                print(resol.format_message_pv2(msg))

    return act_values          
    

def get_data():

    global old_values
    global old_time
    global interval
    global message_count
    
    act_values = load_data()
    act_time = int (time.time())
    if config.simulation:
            message_count +=1
            if message_count >= len (messages_array):
                message_count = 0
            time.sleep(1)
    
    if config.debug:
        print ("Actual values from control:", act_values)
    if (act_values != old_values) and ((act_time - old_time) > config.interval):
        print ("sending actual values to webserver:")
        old_values = act_values
        old_time = act_time
        for channel in  range(len (config.channels)):
            url=config.server_url+config.channels[channel][0]+".json?operation=add&ts="+str(act_time)+"&value="+str(act_values[channel])
            print (url)
            if not config.filewrite:
                requests.post(url)
            else:
                with open(config.filename, "a") as value_file:
                    print(url,file=value_file)

if __name__ == '__main__':

    old_values = []
    old_time=int(time.time())
    signal.signal(signal.SIGINT, signal_handler)
    print ("Resol-Reader . Fetching data from Solar-controller and uplods to middleware")
    
    if not config.simulation:
        ser = serial.Serial(config.port,timeout=0.5)
        if not ser:
            print ("Failed to open serial port")
            sys.exit(-1)
            
        #result = dict()
        login()
    else:
        message_count = 0

    while run_loop:
        get_data()

    try:
        ser.close()
    except:
        pass
    ser = None
    print ("Resol-Reader . done")
